@@ -1,6 +1,6 @@
 import contextlib
 
-from typing import Any, Dict, Generator, Iterable, Optional, Tuple, Union
+from typing import Any, Dict, Generator, Iterable, Optional, Set, Tuple, Union
 
 import torch
 
@@ -291,6 +291,26 @@ class BaseRuntime:
         """
         raise NotImplementedError()
 
+    def map(
+        self,
+        func: CodeBlock,
+        iterable: Iterable[Any],
+        out_keys: Optional[Set[str]] = None,
+        device: Any = "cpu",
+    ) -> Iterable[Any]:
+        """Method called by the user to apply function to iterable efficiently.
+
+        Args:
+            func: The function to be executed
+            iterable: The data
+            out_keys: The output keys that to be moved to the host device
+            device: The torch device that contains the final outputs
+
+        Returns:
+            The result of `func`
+        """
+        raise NotImplementedError()
+
 
 class PyTorchRuntime(BaseRuntime):
     """A collections of callback functions for the devices that PyTorch
@@ -406,8 +426,8 @@ class PyTorchRuntime(BaseRuntime):
             def _scale(x: torch.Tensor) -> torch.Tensor:
                 return self._grad_scaler.scale(x)  # type: ignore[no-any-return]
 
-        if code_block.optimizer is not None:
-            code_block.optimizer.zero_grad()
+        for optimizer in code_block.optimizers:
+            optimizer.zero_grad()
 
         # with autocast
         with _autocast(enabled=self._autocast):
@@ -430,17 +450,39 @@ class PyTorchRuntime(BaseRuntime):
             else:
                 _scale(out[code_block.backprop_from]).backward()  # type: ignore
 
-        if code_block.optimizer is None:
+        if len(code_block.optimizers) == 0:
             return out
 
         if self._grad_scaler is not None:
-            self._grad_scaler.step(code_block.optimizer)
+            # TODO support multiple optimizers with grad scaler
+            assert len(code_block.optimizers) == 1
+            self._grad_scaler.step(code_block.optimizers[0])
             self._grad_scaler.update()
         else:
-            code_block.optimizer.step()
+            for optimizer in code_block.optimizers:
+                optimizer.step()
 
         return out
 
+    def map(
+        self,
+        func: CodeBlock,
+        iterable: Iterable[Any],
+        out_keys: Optional[Set[str]] = None,
+        device: Any = "cpu",
+    ) -> Iterable[Any]:
+        for data in iterable:
+            # TODO overlap computation and data transfer when using CUDA
+            out = func(data)
+            if out_keys is not None:
+                assert isinstance(out, dict)
+                out = {key: out[key] for key in out_keys}
+            if isinstance(out, dict):
+                out = {k: v.to(device) for k, v in out.items()}
+            else:
+                out = out.to(device)
+            yield out
+
 
 def _module_runtime_tag(module: torch.nn.Module) -> Optional[BaseRuntime]:
     return getattr(  # type: ignore[no-any-return]
@@ -467,9 +509,5 @@ def named_runtime_modules(
             for name, sm in module.named_children():
                 yield from named_runtime_modules(sm, name, False, recursive)
     else:
-        if first_level or recursive:
-            for sm in module.children():
-                for descendant in sm.modules():
-                    if _module_runtime_tag(descendant) is not None:
-                        raise ValueError("Runtimes cannot be nested.")
+        # nested runtime tag is ignored
         yield module_name, module
