@@ -1,5 +1,6 @@
-# mypy: ignore-errors
+import typing
 
+import torch
 from torch.nn.parallel import DistributedDataParallel
 
 from pytorch_pfn_extras.nn.parallel import (
@@ -7,7 +8,10 @@ from pytorch_pfn_extras.nn.parallel import (
 )
 
 
-def default_transform_model(n, x):
+_TransformModel = typing.Callable[[str, torch.nn.Module], torch.nn.Module]
+
+
+def default_transform_model(n: str, x: torch.nn.Module) -> torch.nn.Module:
     if isinstance(x, (DistributedDataParallel, PpeDistributedDataParallel)):
         return x.module
     return x
