@@ -1,7 +1,7 @@
 import pytest
 import torch
 
-from tests.pytorch_pfn_extras_tests.dataset_tests.tabular_tests import (
+from pytorch_pfn_extras_tests.dataset_tests.tabular_tests import (
     dummy_dataset,
 )  # NOQA
 
