@@ -6,6 +6,13 @@ import pytest
 import pytorch_pfn_extras as ppe
 
 
+def torch_testing_assert_close(*args, **kwargs):
+    if ppe.requires("1.10.0"):
+        torch.testing.assert_close(*args, **kwargs)
+    else:
+        torch.testing.assert_allclose(*args, **kwargs)
+
+
 class MockRuntime(ppe.runtime.BaseRuntime):
     def __init__(self, device, options):
         super().__init__(device, options)
@@ -14,10 +21,10 @@ class MockRuntime(ppe.runtime.BaseRuntime):
         self._called_module = None
 
     def move_module(self, module):
-        pass
+        return module
 
     def move_tensor(self, tensor):
-        pass
+        return tensor
 
     def convert_batch(self, batch):
         class BatchWrapper:
@@ -36,6 +43,14 @@ class MockRuntime(ppe.runtime.BaseRuntime):
         self._train_epoch_begin_called = True
         self._called_module = module
 
+    def train_epoch_end(self, module):
+        self._train_epoch_end_called = True
+        self._called_module = module
+
+    def train_cleanup(self, module):
+        self._train_cleanup_called = True
+        self._called_module = module
+
     def train_pre_step(self, trainer, module, batch_idx, batch):
         self._train_pre_step_called = True
         self._called_module = module
@@ -56,9 +71,6 @@ class MockRuntime(ppe.runtime.BaseRuntime):
         self._eval_post_step_called = True
         self._called_module = module
 
-    def get_pending_result(self, module, blocking):
-        pass
-
 
 class MockModule(torch.nn.Module):
     def __init__(self):
@@ -88,6 +100,9 @@ class MockLogic(ppe.handler.BaseLogic):
     def train_epoch_begin(self, epoch, models, loader):
         self._train_epoch_begin_called = True
 
+    def train_epoch_end(self, epoch, models):
+        self._train_epoch_end_called = True
+
     def train_step(self, models, optimizers, batch_idx, batch):
         assert batch.converted
         return models['main'](batch)
@@ -128,7 +143,8 @@ class HandlerTester:
                 assert getattr(mod._ppe_runtime, f'_{function}_called')
                 assert mod._ppe_runtime._called_module == mod
             else:
-                assert not hasattr(mod, '_ppe_runtime')
+                if hasattr(mod, '_ppe_runtime'):
+                    assert mod._ppe_runtime._called_module != mod
 
 
 class TestHandlerTrainSync(HandlerTester):
@@ -143,6 +159,16 @@ class TestHandlerTrainSync(HandlerTester):
         handler.train_setup(trainer, [])
         self._assert_called(module, to_move, 'initialize')
 
+    @pytest.mark.parametrize(
+        'to_move', [('self',), ('sm1',), ('sm2',), ('sm1', 'sm2')]
+    )
+    def test_train_cleanup(self, to_move):
+        handler, trainer, _ = self._get_handler()
+        module = trainer.models['main']
+        self._move_modules(module, to_move)
+        handler.train_cleanup(trainer)
+        self._assert_called(module, to_move, 'train_cleanup')
+
     @pytest.mark.parametrize(
         'to_move', [('self',), ('sm1',), ('sm2',), ('sm1', 'sm2')]
     )
@@ -154,13 +180,17 @@ class TestHandlerTrainSync(HandlerTester):
         self._assert_called(module, to_move, 'train_epoch_begin')
         assert logic._train_epoch_begin_called
 
-    def test_train_epoch_end(self):
-        handler, trainer, _ = self._get_handler()
+    @pytest.mark.parametrize(
+        'to_move', [('self',), ('sm1',), ('sm2',), ('sm1', 'sm2')]
+    )
+    def test_train_epoch_end(self, to_move):
+        handler, trainer, logic = self._get_handler()
+        module = trainer.models['main']
+        self._move_modules(module, to_move)
         # Should check that the handler completes
-        assert not handler._async
-        assert not handler.pending_iters
         handler.train_epoch_end(trainer)
-        assert not handler.pending_iters
+        self._assert_called(module, to_move, 'train_epoch_end')
+        assert logic._train_epoch_end_called
 
     @pytest.mark.parametrize(
         'to_move', [('self',), ('sm1',), ('sm2',), ('sm1', 'sm2')]
@@ -248,85 +278,6 @@ class TestHandlerValidationSync(HandlerTester):
         self._assert_called(module, to_move, 'eval_post_step')
 
 
-class AsyncRuntime(MockRuntime):
-    def __init__(self, device, options):
-        super().__init__(device, options)
-        # Returns a result once every 10 items
-        self._period = 10
-        self._count = 0
-
-    def get_pending_result(self, module, blocking):
-        self._count = (self._count + 1) % self._period
-        if self._count == 0 or blocking:
-            return 1
-        return None
-
-
-class TestAsyncHandler:
-    def _get_handler(self, options):
-        ppe.runtime.runtime_registry.register('test_rt', AsyncRuntime)
-        logic = MockLogic()
-        handler = ppe.handler.Handler(
-            logic, AsyncRuntime('test_rt', {}), options
-        )
-        return handler
-
-    def test_train_step_async(self):
-        options = {'eval_report_keys': ['output'], 'async': True}
-        trainer = MockTrainer()
-        handler = self._get_handler(options)
-        ppe.to(trainer.models['main'], 'test_rt')
-        prev_batch_idx = 0
-
-        def callback(batch_idx, outs, is_deferred):
-            nonlocal prev_batch_idx
-            # Check that iterations complete in order
-            assert prev_batch_idx == batch_idx
-            prev_batch_idx += 1
-            assert outs == 1
-
-        for i in range(40):
-            handler.train_step(trainer, i, None, callback)
-        assert prev_batch_idx == 4
-        assert len(handler.pending_iters['main']) == 36
-
-        handler.train_epoch_end(trainer)
-        assert prev_batch_idx == 40
-        assert len(handler.pending_iters['main']) == 0
-
-    def test_eval_step_async(self):
-        options = {'eval_report_keys': ['output'], 'async': True}
-        handler = self._get_handler(options)
-        evaluator = MockEvaluator()
-        ppe.to(evaluator.models['main'], 'test_rt')
-        prev_batch_idx = 0
-
-        def callback(batch_idx, outs, is_deferred):
-            nonlocal prev_batch_idx
-            # Check that iterations complete in order
-            assert prev_batch_idx == batch_idx
-            prev_batch_idx += 1
-            assert outs == 1
-
-        for i in range(40):
-            handler.eval_step(evaluator, i, None, callback)
-
-        assert prev_batch_idx == 4
-        assert len(handler.pending_iters['main']) == 36
-        handler.eval_loop_end(evaluator)
-        assert prev_batch_idx == 40
-        assert len(handler.pending_iters['main']) == 0
-
-    def test_setup_multi_device_split_invalid(self):
-        options = {'eval_report_keys': ['output'], 'async': True}
-        trainer = MockTrainer()
-        handler = self._get_handler(options)
-        ppe.to(trainer.models['main'].sm1, 'test_rt')
-        ppe.to(trainer.models['main'].sm2, 'cpu')
-        with pytest.raises(RuntimeError, match='models splitted'):
-            handler._setup(trainer.models, [], None)
-
-
 @pytest.mark.gpu
 class TestHandlerAutocast:
     @pytest.mark.parametrize('autocast', [True, False])
@@ -334,7 +285,7 @@ class TestHandlerAutocast:
         trainer = MockTrainer()
         logic = ppe.handler.Logic(options={'autocast': autocast})
         handler = ppe.handler.Handler(
-            logic, ppe.runtime.PyTorchRuntime('cuda'), {}
+            logic, ppe.runtime.PyTorchRuntime('cuda', {}), {}
         )
 
         completed = False
@@ -366,13 +317,13 @@ class TestHandlerAutocast:
         assert completed
 
     def test_autocast_not_enabled(self):
-        old_enable = ppe.handler._amp_enabled
+        old_enable = ppe.handler._logic._amp_enabled
         try:
-            ppe.handler._amp_enabled = False
+            ppe.handler._logic._amp_enabled = False
             with pytest.raises(RuntimeError):
                 ppe.handler.Logic(options={'autocast': True})
         finally:
-            ppe.handler._amp_enabled = old_enable
+            ppe.handler._logic._amp_enabled = old_enable
 
 
 class TestLogic:
@@ -398,7 +349,15 @@ class TestLogic:
     def _run_step(self, logic, device):
         input = torch.rand(1, 1).to(device)
         input.requires_grad = True
-        model = torch.nn.Linear(1, 1).to(device)
+
+        class _Module(torch.nn.Linear):
+            def __init__(self) -> None:
+                super().__init__(1, 1)
+
+            def forward(self, x: torch.Tensor) -> torch.Tensor:
+                return super().forward(x).sum()
+
+        model = _Module().to(device)
         models = {'main': model}
         optimizers = {'main': torch.optim.SGD(model.parameters(), 1.0, 0)}
         out = logic.train_step(models, optimizers, 0, input)
@@ -410,8 +369,8 @@ class TestLogic:
         model = models['main']
         assert input.grad is not None
         # The gradient of a linear layer is its transposed weight
-        torch.testing.assert_allclose(input.grad, model.weight.T)
-        torch.testing.assert_allclose(out, model(input))
+        torch_testing_assert_close(input.grad, model.weight.T)
+        torch_testing_assert_close(out, model(input))
 
     @pytest.mark.parametrize(
         'to_backprop',
@@ -452,11 +411,11 @@ class TestLogic:
         grad = torch.zeros(1)
         for val in to_backprop:
             grad = grad + getattr(model, f'l{val}').weight.T
-        torch.testing.assert_allclose(input.grad, grad)
+        torch_testing_assert_close(input.grad, grad)
 
         # Check that logic step does not change the value of weight
         for val in original_parameters:
-            torch.testing.assert_allclose(
+            torch_testing_assert_close(
                 original_parameters[val], getattr(model, f'l{val}').weight)
 
     def test_train_step_backward_nograd(self):
@@ -481,6 +440,27 @@ class TestLogic:
 
         assert outs['0'].grad is None
 
+    def test_train_step_backward_invalid(self):
+        logic = ppe.handler.Logic(options={'backward_outputs': 'abcd'})
+        input = torch.rand(1, 1)
+        input.requires_grad = True
+
+        class _DummyModel(torch.nn.Module):
+            def __init__(self):
+                super().__init__()
+                self.l0 = torch.nn.Linear(1, 1)
+
+            def forward(self, x):
+                return {'0': x}
+
+        model = _DummyModel()
+        models = {'main': model}
+        optimizers = {'main': torch.optim.SGD(model.parameters(), 1.0)}
+        assert input.grad is None
+
+        with pytest.warns(UserWarning, match='backward value: abcd'):
+            logic.train_step(models, optimizers, 0, input)
+
     def test_train_step_optimizers(self):
         logic = ppe.handler.Logic()
         models, optimizers, input, out = self._run_step(logic, 'cpu')
@@ -489,7 +469,7 @@ class TestLogic:
         w_grad = model.weight.grad.clone().detach()
         logic.train_step_optimizers(model, optimizers, 0)
         # Checks that the value was correctly updated
-        torch.testing.assert_allclose(m_weight - w_grad, model.weight.T)
+        torch_testing_assert_close(m_weight - w_grad, model.weight.T)
 
     @pytest.mark.gpu
     def test_grad_scaler(self):
@@ -501,12 +481,12 @@ class TestLogic:
         m_weight = model.weight.clone().detach()
         w_grad = model.weight.grad.clone().detach()
         # The gradient of a linear layer is its transposed weight
-        torch.testing.assert_allclose(input.grad, scaler.scale(model.weight.T))
-        torch.testing.assert_allclose(out, model(input))
+        torch_testing_assert_close(input.grad, scaler.scale(model.weight.T))
+        torch_testing_assert_close(out, model(input))
         logic.train_step_optimizers(model, optimizers, 0)
         # Checks that the value was correctly updated and gradients deescaled
         # before the update
-        torch.testing.assert_allclose(
+        torch_testing_assert_close(
             scaler.scale(m_weight) - w_grad, scaler.scale(model.weight.T))
 
     @pytest.mark.gpu
@@ -515,15 +495,16 @@ class TestLogic:
         with pytest.raises(RuntimeError):
             ppe.handler.Logic(options=options)
 
+    @pytest.mark.gpu
     def test_disabled_grad_scaler(self):
-        old_enable = ppe.handler._amp_enabled
+        old_enable = ppe.handler._logic._amp_enabled
         try:
-            ppe.handler._amp_enabled = False
+            ppe.handler._logic._amp_enabled = False
             options = {'grad_scaler': torch.cuda.amp.GradScaler()}
             with pytest.raises(RuntimeError):
                 ppe.handler.Logic(options=options)
         finally:
-            ppe.handler._amp_enabled = old_enable
+            ppe.handler._logic._amp_enabled = old_enable
 
     def test_train_validation_begin(self):
         logic = ppe.handler.Logic()
@@ -540,4 +521,4 @@ class TestLogic:
         models = {'main': model}
         models['main'].eval()
         out = logic.eval_step(models, 0, input)
-        torch.testing.assert_allclose(out, model(input))
+        torch_testing_assert_close(out, model(input))
