@@ -16,5 +16,4 @@ def test_tensorboard_writing():
         # Check that the file was generated
         for snap in os.listdir(tempd):
             assert '_test' in snap
-        writer._writer.flush()
-        writer._writer.close()
+        writer.finalize()
